%*****************************************************************************80
%
%% Evaluates the objective function for problem 2.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = BiggsEXP6(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Biggs EXP6 function' ;

  case {'n','size'}
    RES = 6 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 1.0, 2.0, 1.0, 1.0, 1.0, 1.0 ]' ;

  case 'solution'
    RES = [ 1.0, 10.0, 1.0, 5.0, 4.0, 3.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Biggs EXP6 function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = 0.0;
  for i = 1 : 13
    c = - i / 10.0;
    fi = x(3)     * exp ( c * x(1) )         - x(4) * exp ( c * x(2) ) ...
       + x(6)     * exp ( c * x(5) )         -        exp ( c ) ...
       + 5.0  * exp ( 10.0 * c ) - 3.0  * exp ( 4.0 * c );
    f = f + fi * fi;
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 6 );

  for i = 1 : 13
    c = - i / 10.0;
    fi =        x(3) * exp ( c * x(1) )         - x(4) * exp ( c * x(2) ) ...
          +     x(6) * exp ( c * x(5) )         -        exp ( c ) ...
          +  5.0 * exp ( 10.0 * c ) -  3.0 * exp ( 4.0 * c );

    dfdx1 =     c    * x(3) * exp ( c * x(1) );
    dfdx2 =   - c    * x(4) * exp ( c * x(2) );
    dfdx3 =                   exp ( c * x(1) );
    dfdx4 =   -               exp ( c * x(2) );
    dfdx5 =     c    * x(6) * exp ( c * x(5) );
    dfdx6 =                   exp ( c * x(5) );

    g(1) = g(1) + 2.0 * fi * dfdx1;
    g(2) = g(2) + 2.0 * fi * dfdx2;
    g(3) = g(3) + 2.0 * fi * dfdx3;
    g(4) = g(4) + 2.0 * fi * dfdx4;
    g(5) = g(5) + 2.0 * fi * dfdx5;
    g(6) = g(6) + 2.0 * fi * dfdx6;
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 6, 6 );

  for i = 1 : 13

    c = - i / 10.0;

    fi =   x(3) * exp ( c * x(1) )  - x(4) * exp ( c * x(2) ) ...
          + x(6) * exp ( c * x(5) ) -        exp ( c ) ...
          + 5.0 * exp ( 10.0 * c ) -  3.0 * exp ( 4.0 * c );

    dfdx1 =     c     * x(3) * exp ( c * x(1) );
    d2fdx11 =   c * c * x(3) * exp ( c * x(1) );
    d2fdx13 =   c     *        exp ( c * x(1) );
    dfdx2 =   - c     * x(4) * exp ( c * x(2) );
    d2fdx22 = - c * c * x(4) * exp ( c * x(2) );
    d2fdx24 = - c     *        exp ( c * x(2) );
    dfdx3 =                    exp ( c * x(1) );
    d2fdx31 =   c     *        exp ( c * x(1) );
    dfdx4 =   -                exp ( c * x(2) );
    d2fdx42 = - c     *        exp ( c * x(2) );
    dfdx5 =     c     * x(6) * exp ( c * x(5) );
    d2fdx55 =   c * c * x(6) * exp ( c * x(5) );
    d2fdx56 =   c     *        exp ( c * x(5) );
    dfdx6 =                    exp ( c * x(5) );
    d2fdx65 =   c     *        exp ( c * x(5) );

    h(1,1) = h(1,1) + 2.0 * dfdx1 * dfdx1 + 2.0 * fi * d2fdx11;
    h(1,2) = h(1,2) + 2.0 * dfdx2 * dfdx1;
    h(1,3) = h(1,3) + 2.0 * dfdx3 * dfdx1 + 2.0 * fi * d2fdx13;
    h(1,4) = h(1,4) + 2.0 * dfdx4 * dfdx1;
    h(1,5) = h(1,5) + 2.0 * dfdx5 * dfdx1;
    h(1,6) = h(1,6) + 2.0 * dfdx6 * dfdx1;

    h(2,1) = h(2,1) + 2.0 * dfdx1 * dfdx2;
    h(2,2) = h(2,2) + 2.0 * dfdx2 * dfdx2 + 2.0 * fi * d2fdx22;
    h(2,3) = h(2,3) + 2.0 * dfdx3 * dfdx2;
    h(2,4) = h(2,4) + 2.0 * dfdx4 * dfdx2 + 2.0 * fi * d2fdx24;
    h(2,5) = h(2,5) + 2.0 * dfdx5 * dfdx2;
    h(2,6) = h(2,6) + 2.0 * dfdx6 * dfdx2;

    h(3,1) = h(3,1) + 2.0 * dfdx1 * dfdx3 + 2.0 * fi * d2fdx31;
    h(3,2) = h(3,2) + 2.0 * dfdx2 * dfdx3;
    h(3,3) = h(3,3) + 2.0 * dfdx3 * dfdx3;
    h(3,4) = h(3,4) + 2.0 * dfdx4 * dfdx3;
    h(3,5) = h(3,5) + 2.0 * dfdx5 * dfdx3;
    h(3,6) = h(3,6) + 2.0 * dfdx6 * dfdx3;

    h(4,1) = h(4,1) + 2.0 * dfdx1 * dfdx4;
    h(4,2) = h(4,2) + 2.0 * dfdx2 * dfdx4 + 2.0 * fi * d2fdx42;
    h(4,3) = h(4,3) + 2.0 * dfdx3 * dfdx4;
    h(4,4) = h(4,4) + 2.0 * dfdx4 * dfdx4;
    h(4,5) = h(4,5) + 2.0 * dfdx5 * dfdx4;
    h(4,6) = h(4,6) + 2.0 * dfdx6 * dfdx4;

    h(5,1) = h(5,1) + 2.0 * dfdx1 * dfdx5;
    h(5,2) = h(5,2) + 2.0 * dfdx2 * dfdx5;
    h(5,3) = h(5,3) + 2.0 * dfdx3 * dfdx5;
    h(5,4) = h(5,4) + 2.0 * dfdx4 * dfdx5;
    h(5,5) = h(5,5) + 2.0 * dfdx5 * dfdx5 + 2.0 * fi * d2fdx55;
    h(5,6) = h(5,6) + 2.0 * dfdx6 * dfdx5 + 2.0 * fi * d2fdx56;

    h(6,1) = h(6,1) + 2.0 * dfdx1 * dfdx6;
    h(6,2) = h(6,2) + 2.0 * dfdx2 * dfdx6;
    h(6,3) = h(6,3) + 2.0 * dfdx3 * dfdx6;
    h(6,4) = h(6,4) + 2.0 * dfdx4 * dfdx6;
    h(6,5) = h(6,5) + 2.0 * dfdx5 * dfdx6 + 2.0 * fi * d2fdx65;
    h(6,6) = h(6,6) + 2.0 * dfdx6 * dfdx6;

  end
end
