%*****************************************************************************80
%
%% Evaluates the objective function for problem 10.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = BrownAndDennis(what,x)
  global Nf_eval Ng_eval Nh_eval ;

  switch what

  case {'name','title'}
    RES = 'The Brown and Dennis Function.' ;

  case {'n','size'}
    RES = 4 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 25.0, 5.0, -5.0, -1.0 ]';

  case 'solution'
    RES = [ -11.5844, 13.1999, -0.406200, 0.240998 ] ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Brown and Dennis Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = 0.0;
  for i = 1 : 20
    c = i / 5.0;
    f1 = x(1) + c * x(2) - exp ( c );
    f2 = x(3) + sin ( c ) * x(4) - cos ( c );
    f = f + f1^4 + 2.0 * f1 * f1 * f2 * f2 + f2^4;
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 4 );

  for i = 1 : 20
    c = i / 5.0;

    f1 = x(1) + c * x(2) - exp ( c );
    f2 = x(3) + sin ( c ) * x(4) - cos ( c );

    df1dx1 = 1.0;
    df1dx2 = c;
    df2dx3 = 1.0;
    df2dx4 = sin ( c );

    g(1) = g(1) + 4.0 * ( f1^3 * df1dx1 + f1 * f2 * f2 * df1dx1 );
    g(2) = g(2) + 4.0 * ( f1^3 * df1dx2 + f1 * f2 * f2 * df1dx2 );
    g(3) = g(3) + 4.0 * ( f1 * f1 * f2 * df2dx3 + f2^3 * df2dx3 );
    g(4) = g(4) + 4.0 * ( f1 * f1 * f2 * df2dx4 + f2^3 * df2dx4 );

  end
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 4, 4 );

  for i = 1 : 20

    c = i / 5.0;

    f1 = x(1) + c * x(2) - exp ( c );
    f2 = x(3) + sin ( c ) * x(4) - cos ( c );

    df1dx1 = 1.0;
    df1dx2 = c;
    df2dx3 = 1.0;
    df2dx4 = sin ( c );

    h(1,1) = h(1,1) + 12.0 * f1 * f1 * df1dx1 * df1dx1 ...
                    +  4.0 * f2 * f2 * df1dx1 * df1dx1;
    h(1,2) = h(1,2) + 12.0 * f1 * f1 * df1dx1 * df1dx2 ...
                    +  4.0 * f2 * f2 * df1dx1 * df1dx2;
    h(1,3) = h(1,3) +  8.0 * f1 * f2 * df1dx1 * df2dx3;
    h(1,4) = h(1,4) +  8.0 * f1 * f2 * df1dx1 * df2dx4;

    h(2,1) = h(2,1) + 12.0 * f1 * f1 * df1dx2 * df1dx1 ...
                    +  4.0 * f2 * f2 * df1dx2 * df1dx1;
    h(2,2) = h(2,2) + 12.0 * f1 * f1 * df1dx2 * df1dx2 ...
                    +  4.0 * f2 * f2 * df1dx2 * df1dx1;
    h(2,3) = h(2,3) +  8.0 * f1 * f2 * df1dx2 * df2dx3;
    h(2,4) = h(2,4) +  8.0 * f1 * f2 * df1dx2 * df2dx4;

    h(3,1) = h(3,1) +  8.0 * f1 * f2 * df2dx3 * df1dx1;
    h(3,2) = h(3,2) +  8.0 * f1 * f2 * df2dx3 * df1dx2;
    h(3,3) = h(3,3) +  4.0 * f1 * f1 * df2dx3 * df2dx3 ...
                    + 12.0 * f2 * f2 * df2dx3 * df2dx3;
    h(3,4) = h(3,4) +  4.0 * f1 * f1 * df2dx4 * df2dx3 ...
                    + 12.0 * f2 * f2 * df2dx3 * df2dx4;

    h(4,1) = h(4,1) +  8.0 * f1 * f2 * df2dx4 * df1dx1;
    h(4,2) = h(4,2) +  8.0 * f1 * f2 * df2dx4 * df1dx2;
    h(4,3) = h(4,3) +  4.0 * f1 * f1 * df2dx3 * df2dx4 ...
                    + 12.0 * f2 * f2 * df2dx4 * df2dx3;
    h(4,4) = h(4,4) +  4.0 * f1 * f1 * df2dx4 * df2dx4 ...
                    + 12.0 * f2 * f2 * df2dx4 * df2dx4;

  end

end
