%*****************************************************************************80
%
%% Evaluates the objective function for problem 38.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Colville( what, x )
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Colville Polynomial.' ;

  case {'n','size'}
    RES = 4 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.5, 1.0, -0.5, -1.0 ]' ;

  case 'solution'
    RES = ones(4,1) ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Colville Polynomial, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = 100.0 * ( x(2) - x(1)^2 )^2 ...
    + ( 1.0 - x(1) )^2 ...
    + 90.0 * ( x(4) - x(3)^2 )^2 ...
    + ( 1.0 - x(3) )^2 ...
    + 10.1 * ( ( x(2) - 1.0 )^2 + ( x(4) - 1.0 )^2 ) ...
    + 19.8 * ( x(2) - 1.0 ) * ( x(4) - 1.0 );
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 4 );

  g(1) = 400.0 * x(1)^3 - 400.0 * x(2) * x(1) + 2.0 * x(1) - 2.0;
  g(2) = -200.0 * x(1)^2 + 220.2 * x(2) + 19.8 * x(4) - 40.0;
  g(3) = -360.0 * x(3) * x(4) + 360.0 * x(3)^3 + 2.0 * x(3) - 2.0;
  g(4) = + 180.0 * x(4) - 180.0 * x(3)^2 + 20.2 * x(4) + 19.8 * x(2) - 40.0;

end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 4, 4 );
  h(1,1) = 1200.0 * x(1)^2 - 400.0 * x(2) + 2.0;
  h(1,2) = - 400.0 * x(1);

  h(2,1) = -400.0 * x(1);
  h(2,2) = 220.2;
  h(2,4) = 19.8;

  h(3,3) = -360.0 * x(4) + 1080.0 * x(3)^2 + 2.0;
  h(3,4) = - 360.0 * x(3);

  h(4,2) = 19.8;
  h(4,3) = - 360.0 * x(3);
  h(4,4) = 200.2;
end
