%*****************************************************************************80
%
% Evaluates the objective function for problem 20.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = GregoryAndKarney( what, x )
  global Nf_eval Ng_eval Nh_eval GregoryAndKarney_n ;

  if isempty(GregoryAndKarney_n)
    GregoryAndKarney_n = 2 ;
  end

  switch what

  case 'setup'
    if x < 1 
      error(['GregoryAndKarney(''setup'', ' x ') argument must be >= 1']) ; 
    end
    GregoryAndKarney_n = x ;

  case {'name','title'}
    RES = 'The Gregory and Karney Tridiagonal Matrix Function.' ;

  case {'n','size'}
    RES = GregoryAndKarney_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = zeros( GregoryAndKarney_n, 1 ) ;

  case 'solution'
    n = GregoryAndKarney_n ;
    for i = 1:n
      RES(i) = n + 1 - i;
    end

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Gregory and Karney Tridiagonal Matrix Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  global GregoryAndKarney_n ;
  n = GregoryAndKarney_n ;
  f = x(1) * x(1) + 2.0 * dot( x, x );
  for i = 1 : n - 1
    f = f - 2.0 * x(i) * x(i+1);
  end
  f = f - 2.0 * x(1);
end

%*****************************************************************************80
function g = eval_g ( x )
  global GregoryAndKarney_n ;
  n = GregoryAndKarney_n ;
  g = zeros ( 1, n );

  for i = 1 : n

    if ( i == 1 )
      g(i) = x(i);
    else
      g(i) = 2.0 * x(i);
    end

    if ( 1 < i )
      g(i) = g(i) - x(i-1);
    end

    if ( i < n )
      g(i) = g(i) - x(i+1);
    end

  end

  g(1) = g(1) - 2.0;
end

%*****************************************************************************80
function h = eval_h ( x )
  global GregoryAndKarney_n ;
  n = GregoryAndKarney_n ;
  h = zeros ( n, n );

  for i = 1 : n
    for j = 1 : n
      if ( i == j )
        if ( i == 1 )
          h(i,j) = 2.0;
        else
          h(i,j) = 4.0;
        end
      elseif ( i == j + 1 || i == j - 1 )
        h(i,j) = - 2.0;
      else
        h(i,j) = 0.0;
      end
    end
  end
end
