%*****************************************************************************80
%
%% Evaluates the objective function for problem 12.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Gulf(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Gulf R&D Function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 40.0, 20.0, 1.20 ]' ;

  case 'solution'
    RES = [ 50.0, 25.0, 1.5 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['Gulf R&D function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = 0.0;
  for i = 1 : 99
    arg = i / 100.0;
    r = abs ( ( - 50.0 * log ( arg ) )^( 2.0 / 3.0 ) + 25.0 - x(2) );
    t = exp ( - r^x(3) / x(1) ) - arg;
    f = f + t * t;
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 3 );

  for i = 1 : 99

    arg = i / 100.0;
    r = abs ( ( - 50.0 * log ( arg ) )^( 2.0 / 3.0 ) + 25.0 - x(2) );
    t1 = r^x(3) / x(1);
    t2 = exp ( - t1 );

    t = exp ( - r^x(3) / x(1) ) - arg;

    g(1) = g(1) + 2.0 * t * t1 * t2 / x(1);
    g(2) = g(2) + 2.0 * t * t1 * t2 * x(3) / r;
    g(3) = g(3) - 2.0 * t * t1 * t2 * log ( r );

  end
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 3, 3 );

  d1 = 2.0 / 3.0;

  for i = 1 : 99

    arg = i / 100.0;
    r = ( - 50.0 * log ( arg ) )^d1 + 25.0 - x(2);
    t1 = abs ( r )^x(3) / x(1);
    t2 = exp ( - t1 );
    t3 = t1 * t2 * ( t1 * t2 + ( t1 - 1.0 ) * ( t2 - arg ) );
    t = t1 * t2 * ( t2 - arg );
    logr = log ( abs ( r ) );

    h(1,1) = h(1,1) + 2.0 * t3 - 2.0 * t;
    h(1,2) = h(1,2) + 2.0 * t3 / r;
    h(1,3) = h(1,3) - 2.0 * t3 * logr;

    h(2,1) = h(2,1) + 2.0 * t3 / r;
    h(2,2) = h(2,2) + 2.0 * ( t + x(3) * t3 ) / r / r;
    h(2,3) = h(2,3) + 2.0 * ( t - x(3) * t3 * logr ) / r;

    h(3,1) = h(3,1) - 2.0 * t3 * logr;
    h(3,2) = h(3,2) + 2.0 * ( t - x(3) * t3 * logr ) / r;
    h(3,3) = h(3,3) + 2.0 * t3 * logr * logr;

  end

  h(1,1) = ( h(1,1) / x(1) ) / x(1);
  h(1,2) = h(1,2) * x(3) / x(1);
  h(1,3) = h(1,3) / x(1);

  h(2,1) = h(2,1) * x(3) / x(1);
  h(2,2) = h(2,2) * x(3);

  h(3,1) = h(3,1) / x(1);
end
