%
% Han-Sun-Han-SAMPAJO 2005 function test
%
% what = 'f'  compute the Han-Sun-Han-SAMPAJO function
% what = 'Df' compute the Gradient of the Han-Sun-Han-SAMPAJO function
% what = 'DDf' compute the Hessian of the Han-Sun-Han-SAMPAJO function
%
function RES = Gulf(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'Han-Sun-Han-SAMPAJO.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 5 ; 3 ] ;

  case 'solution'
    RES = [] ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h( x ) ;

  otherwise
    error(['Han-Sun-Han-SAMPAJO, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f = x(1)^8 + x(1)^2 + x(1)^2 * x(2)^ 2 + exp(x(2)^2);
end

%*****************************************************************************80
function g = eval_g ( x )
  g   = zeros ( 1, 2 );
  t1  = (x(1) ^ 2);
  t3  = (t1 ^ 2);
  t7  = x(2) ^ 2;
  t12 = exp(t7);
  g(1) = (8 * t3 * t1 * x(1)) + (2 * x(1)) + 0.2e1 * x(1) * t7;
  g(2) = (2 * t1 * x(2)) + 0.2e1 * x(2) * t12;
end

%*****************************************************************************80
function h = eval_h ( x )
  h   = zeros( 2, 2 ) ;
  t1  = x(1)^2 ;
  t2  = t1^2 ;
  t5  = x(2)^2;
  t9  = 4 * x(1) * x(2);
  t11 = exp(t5) ;
  h(1,1) = (56 * t2 * t1) + 2 + 2 * t5;
  h(1,2) = t9;
  h(2,1) = t9;
  h(2,2) = (2 * t1) + 2 * t11 + 4 * t5 * t11;
end
