%*****************************************************************************80
%
%% Evaluates the objective function for problem 4.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = PowellBadlyScaled(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Powell badly scaled function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 0.0, 1.0 ]' ;

  case 'solution'
    RES = [ 1.098159E-05, 9.106146 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['Powell badly scaled function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f1 = 10000.0 * x(1) * x(2) - 1.0;
  f2 = exp ( - x(1) ) + exp ( - x(2) ) - 1.0001;
  f  = f1 * f1 + f2 * f2;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 2 );

  f1     = 10000.0 * x(1) * x(2) - 1.0;
  df1dx1 = 10000.0 * x(2);
  df1dx2 = 10000.0 * x(1);

  f2     = exp ( - x(1) ) + exp ( - x(2) ) - 1.0001;
  df2dx1 = - exp ( - x(1) );
  df2dx2 = - exp ( - x(2) );

  g(1) = 2.0 * f1 * df1dx1 + 2.0 * f2 * df2dx1;
  g(2) = 2.0 * f1 * df1dx2 + 2.0 * f2 * df2dx2;
end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 2, 2 );

  f1       = 10000.0 * x(1) * x(2) - 1.0;
  df1dx1   = 10000.0 * x(2);
  df1dx2   = 10000.0 * x(1);
  d2f1dx12 = 10000.0;
  d2f1dx21 = 10000.0;

  f2       = exp ( - x(1) ) + exp ( - x(2) ) - 1.0001;
  df2dx1   = - exp ( - x(1) );
  df2dx2   = - exp ( - x(2) );
  d2f2dx11 = exp ( - x(1) );
  d2f2dx22 = exp ( - x(2) );

  h(1,1) = 2.0 * df1dx1 * df1dx1 + 2.0 * f2 * d2f2dx11 + 2.0 * df2dx1 * df2dx1;
  h(1,2) = 2.0 * f1 * d2f1dx12 + 2.0 * df1dx2 * df1dx1 + 2.0 * df2dx2 * df2dx1;
  h(2,1) = 2.0 * f1 * d2f1dx21 + 2.0 * df1dx2 * df1dx1 + 2.0 * df2dx2 * df2dx1;
  h(2,2) = 2.0 * df1dx2 * df1dx2 + 2.0 * f2 * d2f2dx22 + 2.0 * df2dx2 * df2dx2;
end
