%*****************************************************************************80
%
%% Evaluates the objective function for problem 2.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSBiggsEXP6(what,x)
  global NF_eval NJF_eval ;
  switch what

  case {'name','title'}
    RES = 'The Biggs EXP6 function' ;

  case {'n','size'}
    RES = 6 ;

  case 'm'
    RES = 13 ;

  case {'init','start'}
    RES =  [ 1.0, 2.0, 1.0, 1.0, 1.0, 1.0 ]' ;

  case 'solution'
    RES = [ 1.0, 10.0, 1.0, 5.0, 4.0, 3.0 ]' ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The Biggs EXP6 function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  F = zeros( 13, 1) ;
  t2 = exp(-x(1) / 0.10e2);
  t5 = exp(-x(2) / 0.10e2);
  t8 = exp(-x(5) / 0.10e2);
  t10 = exp(-0.1e1 / 0.10e2);
  t11 = exp(-0.1e1);
  t13 = exp(-0.2e1 / 0.5e1);
  t17 = exp(-x(1) / 0.5e1);
  t20 = exp(-x(2) / 0.5e1);
  t23 = exp(-x(5) / 0.5e1);
  t25 = exp(-0.1e1 / 0.5e1);
  t26 = exp(-0.2e1);
  t28 = exp(-0.4e1 / 0.5e1);
  t32 = exp(-0.3e1 / 0.10e2 * x(1));
  t35 = exp(-0.3e1 / 0.10e2 * x(2));
  t38 = exp(-0.3e1 / 0.10e2 * x(5));
  t40 = exp(-0.3e1 / 0.10e2);
  t41 = exp(-0.3e1);
  t43 = exp(-0.6e1 / 0.5e1);
  t47 = exp(-0.2e1 / 0.5e1 * x(1));
  t50 = exp(-0.2e1 / 0.5e1 * x(2));
  t53 = exp(-0.2e1 / 0.5e1 * x(5));
  t55 = exp(-0.4e1);
  t57 = exp(-0.8e1 / 0.5e1);
  t61 = exp(-x(1) / 0.2e1);
  t64 = exp(-x(2) / 0.2e1);
  t67 = exp(-x(5) / 0.2e1);
  t69 = exp(-0.1e1 / 0.2e1);
  t70 = exp(-0.5e1);
  t75 = exp(-0.3e1 / 0.5e1 * x(1));
  t78 = exp(-0.3e1 / 0.5e1 * x(2));
  t81 = exp(-0.3e1 / 0.5e1 * x(5));
  t83 = exp(-0.3e1 / 0.5e1);
  t84 = exp(-0.6e1);
  t86 = exp(-0.12e2 / 0.5e1);
  t90 = exp(-0.7e1 / 0.10e2 * x(1));
  t93 = exp(-0.7e1 / 0.10e2 * x(2));
  t96 = exp(-0.7e1 / 0.10e2 * x(5));
  t98 = exp(-0.7e1 / 0.10e2);
  t99 = exp(-0.7e1);
  t101 = exp(-0.14e2 / 0.5e1);
  t105 = exp(-0.4e1 / 0.5e1 * x(1));
  t108 = exp(-0.4e1 / 0.5e1 * x(2));
  t111 = exp(-0.4e1 / 0.5e1 * x(5));
  t113 = exp(-0.8e1);
  t115 = exp(-0.16e2 / 0.5e1);
  t119 = exp(-0.9e1 / 0.10e2 * x(1));
  t122 = exp(-0.9e1 / 0.10e2 * x(2));
  t125 = exp(-0.9e1 / 0.10e2 * x(5));
  t127 = exp(-0.9e1 / 0.10e2);
  t128 = exp(-0.9e1);
  t130 = exp(-0.18e2 / 0.5e1);
  t133 = exp(-x(1));
  t135 = exp(-x(2));
  t137 = exp(-x(5));
  t139 = exp(-0.10e2);
  t144 = exp(-0.11e2 / 0.10e2 * x(1));
  t147 = exp(-0.11e2 / 0.10e2 * x(2));
  t150 = exp(-0.11e2 / 0.10e2 * x(5));
  t152 = exp(-0.11e2 / 0.10e2);
  t153 = exp(-0.11e2);
  t155 = exp(-0.22e2 / 0.5e1);
  t159 = exp(-0.6e1 / 0.5e1 * x(1));
  t162 = exp(-0.6e1 / 0.5e1 * x(2));
  t165 = exp(-0.6e1 / 0.5e1 * x(5));
  t167 = exp(-0.12e2);
  t169 = exp(-0.24e2 / 0.5e1);
  t173 = exp(-0.13e2 / 0.10e2 * x(1));
  t176 = exp(-0.13e2 / 0.10e2 * x(2));
  t179 = exp(-0.13e2 / 0.10e2 * x(5));
  t181 = exp(-0.13e2 / 0.10e2);
  t182 = exp(-0.13e2);
  t184 = exp(-0.26e2 / 0.5e1);
  F(1) = x(3) * t2 - x(4) * t5 + x(6) * t8 - t10 + 0.5e1 * t11 - 0.3e1 * t13;
  F(2) = x(3) * t17 - x(4) * t20 + x(6) * t23 - t25 + 0.5e1 * t26 - 0.3e1 * t28;
  F(3) = x(3) * t32 - x(4) * t35 + x(6) * t38 - t40 + 0.5e1 * t41 - 0.3e1 * t43;
  F(4) = x(3) * t47 - x(4) * t50 + x(6) * t53 - t13 + 0.5e1 * t55 - 0.3e1 * t57;
  F(5) = x(3) * t61 - x(4) * t64 + x(6) * t67 - t69 + 0.5e1 * t70 - 0.3e1 * t26;
  F(6) = x(3) * t75 - x(4) * t78 + x(6) * t81 - t83 + 0.5e1 * t84 - 0.3e1 * t86;
  F(7) = x(3) * t90 - x(4) * t93 + x(6) * t96 - t98 + 0.5e1 * t99 - 0.3e1 * t101;
  F(8) = x(3) * t105 - x(4) * t108 + x(6) * t111 - t28 + 0.5e1 * t113 - 0.3e1 * t115;
  F(9) = x(3) * t119 - x(4) * t122 + x(6) * t125 - t127 + 0.5e1 * t128 - 0.3e1 * t130;
  F(10) = x(3) * t133 - x(4) * t135 + x(6) * t137 - t11 + 0.5e1 * t139 - 0.3e1 * t55;
  F(11) = x(3) * t144 - x(4) * t147 + x(6) * t150 - t152 + 0.5e1 * t153 - 0.3e1 * t155;
  F(12) = x(3) * t159 - x(4) * t162 + x(6) * t165 - t43 + 0.5e1 * t167 - 0.3e1 * t169;
  F(13) = x(3) * t173 - x(4) * t176 + x(6) * t179 - t181 + 0.5e1 * t182 - 0.3e1 * t184;
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros( 13, 6) ;

  t2 = exp(-x(1) / 0.10e2);
  t6 = exp(-x(2) / 0.10e2);
  t10 = exp(-x(5) / 0.10e2);
  t14 = exp(-x(1) / 0.5e1);
  t18 = exp(-x(2) / 0.5e1);
  t22 = exp(-x(5) / 0.5e1);
  t26 = exp(-0.3e1 / 0.10e2 * x(1));
  t30 = exp(-0.3e1 / 0.10e2 * x(2));
  t34 = exp(-0.3e1 / 0.10e2 * x(5));
  t38 = exp(-0.2e1 / 0.5e1 * x(1));
  t42 = exp(-0.2e1 / 0.5e1 * x(2));
  t46 = exp(-0.2e1 / 0.5e1 * x(5));
  t50 = exp(-x(1) / 0.2e1);
  t54 = exp(-x(2) / 0.2e1);
  t58 = exp(-x(5) / 0.2e1);
  t62 = exp(-0.3e1 / 0.5e1 * x(1));
  t66 = exp(-0.3e1 / 0.5e1 * x(2));
  t70 = exp(-0.3e1 / 0.5e1 * x(5));
  t74 = exp(-0.7e1 / 0.10e2 * x(1));
  t78 = exp(-0.7e1 / 0.10e2 * x(2));
  t82 = exp(-0.7e1 / 0.10e2 * x(5));
  t86 = exp(-0.4e1 / 0.5e1 * x(1));
  t90 = exp(-0.4e1 / 0.5e1 * x(2));
  t94 = exp(-0.4e1 / 0.5e1 * x(5));
  t98 = exp(-0.9e1 / 0.10e2 * x(1));
  t102 = exp(-0.9e1 / 0.10e2 * x(2));
  t106 = exp(-0.9e1 / 0.10e2 * x(5));
  t109 = exp(-x(1));
  t111 = exp(-x(2));
  t113 = exp(-x(5));
  t116 = exp(-0.11e2 / 0.10e2 * x(1));
  t120 = exp(-0.11e2 / 0.10e2 * x(2));
  t124 = exp(-0.11e2 / 0.10e2 * x(5));
  t128 = exp(-0.6e1 / 0.5e1 * x(1));
  t132 = exp(-0.6e1 / 0.5e1 * x(2));
  t136 = exp(-0.6e1 / 0.5e1 * x(5));
  t140 = exp(-0.13e2 / 0.10e2 * x(1));
  t144 = exp(-0.13e2 / 0.10e2 * x(2));
  t148 = exp(-0.13e2 / 0.10e2 * x(5));
  JF(1,1) = -x(3) * t2 / 0.10e2;
  JF(1,2) = x(4) * t6 / 0.10e2;
  JF(1,3) = t2;
  JF(1,4) = -t6;
  JF(1,5) = -x(6) * t10 / 0.10e2;
  JF(1,6) = t10;
  JF(2,1) = -x(3) * t14 / 0.5e1;
  JF(2,2) = x(4) * t18 / 0.5e1;
  JF(2,3) = t14;
  JF(2,4) = -t18;
  JF(2,5) = -x(6) * t22 / 0.5e1;
  JF(2,6) = t22;
  JF(3,1) = -0.3e1 / 0.10e2 * x(3) * t26;
  JF(3,2) = 0.3e1 / 0.10e2 * x(4) * t30;
  JF(3,3) = t26;
  JF(3,4) = -t30;
  JF(3,5) = -0.3e1 / 0.10e2 * x(6) * t34;
  JF(3,6) = t34;
  JF(4,1) = -0.2e1 / 0.5e1 * x(3) * t38;
  JF(4,2) = 0.2e1 / 0.5e1 * x(4) * t42;
  JF(4,3) = t38;
  JF(4,4) = -t42;
  JF(4,5) = -0.2e1 / 0.5e1 * x(6) * t46;
  JF(4,6) = t46;
  JF(5,1) = -x(3) * t50 / 0.2e1;
  JF(5,2) = x(4) * t54 / 0.2e1;
  JF(5,3) = t50;
  JF(5,4) = -t54;
  JF(5,5) = -x(6) * t58 / 0.2e1;
  JF(5,6) = t58;
  JF(6,1) = -0.3e1 / 0.5e1 * x(3) * t62;
  JF(6,2) = 0.3e1 / 0.5e1 * x(4) * t66;
  JF(6,3) = t62;
  JF(6,4) = -t66;
  JF(6,5) = -0.3e1 / 0.5e1 * x(6) * t70;
  JF(6,6) = t70;
  JF(7,1) = -0.7e1 / 0.10e2 * x(3) * t74;
  JF(7,2) = 0.7e1 / 0.10e2 * x(4) * t78;
  JF(7,3) = t74;
  JF(7,4) = -t78;
  JF(7,5) = -0.7e1 / 0.10e2 * x(6) * t82;
  JF(7,6) = t82;
  JF(8,1) = -0.4e1 / 0.5e1 * x(3) * t86;
  JF(8,2) = 0.4e1 / 0.5e1 * x(4) * t90;
  JF(8,3) = t86;
  JF(8,4) = -t90;
  JF(8,5) = -0.4e1 / 0.5e1 * x(6) * t94;
  JF(8,6) = t94;
  JF(9,1) = -0.9e1 / 0.10e2 * x(3) * t98;
  JF(9,2) = 0.9e1 / 0.10e2 * x(4) * t102;
  JF(9,3) = t98;
  JF(9,4) = -t102;
  JF(9,5) = -0.9e1 / 0.10e2 * x(6) * t106;
  JF(9,6) = t106;
  JF(10,1) = -x(3) * t109;
  JF(10,2) = x(4) * t111;
  JF(10,3) = t109;
  JF(10,4) = -t111;
  JF(10,5) = -x(6) * t113;
  JF(10,6) = t113;
  JF(11,1) = -0.11e2 / 0.10e2 * x(3) * t116;
  JF(11,2) = 0.11e2 / 0.10e2 * x(4) * t120;
  JF(11,3) = t116;
  JF(11,4) = -t120;
  JF(11,5) = -0.11e2 / 0.10e2 * x(6) * t124;
  JF(11,6) = t124;
  JF(12,1) = -0.6e1 / 0.5e1 * x(3) * t128;
  JF(12,2) = 0.6e1 / 0.5e1 * x(4) * t132;
  JF(12,3) = t128;
  JF(12,4) = -t132;
  JF(12,5) = -0.6e1 / 0.5e1 * x(6) * t136;
  JF(12,6) = t136;
  JF(13,1) = -0.13e2 / 0.10e2 * x(3) * t140;
  JF(13,2) = 0.13e2 / 0.10e2 * x(4) * t144;
  JF(13,3) = t140;
  JF(13,4) = -t144;
  JF(13,5) = -0.13e2 / 0.10e2 * x(6) * t148;
  JF(13,6) = t148;
end
