%*****************************************************************************80
%
%% Evaluates the objective function for problem 5.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSBox3(what,x)
  global NF_eval NJF_eval ;
  switch what

  case {'name','title'}
    RES = 'The Box 3-dimensional function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 10 ;

  case {'init','start'}
    RES = [ 0.0, 10.0, 5.0 ]' ;

  case 'solution'
    RES = [ 1.0, 10.0, 1.0 ]' ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The Box 3-dimensional function:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  F = zeros(10,1) ;
  for i = 1 : 10
    c = - i / 10.0;
    F(i) = exp ( c * x(1) ) - exp ( c * x(2) ) - x(3) * ( exp ( c ) - exp ( 10.0 * c ) );
  end
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros ( 10, 3 ) ;
  tmp = 13/10 ;
  for i = 1 : 10
    c = - i / 10.0 ;
    JF(i,1) = -tmp * exp(-tmp * x(1));
    JF(i,2) =  tmp * exp(-tmp * x(2));
    JF(i,3) = -exp(-tmp) + exp(-13);
  end
end
