%*****************************************************************************80
%
%% Evaluates the objective function for problem 3.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSGauss(what,x)
  global NF_eval NJF_eval  ;
  switch what

  case {'name','title'}
    RES = 'The Gaussian function.' ;

  case {'n','size'}
    RES = 3 ;

  case 'm'
    RES = 15 ;

  case {'init','start'}
    RES = [ 0.4, 1.0, 0.0 ]' ;

  case 'solution'
    RES = [ 0.0, 0.0, 0.0 ]' ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The Gaussian function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function y = yvec()

  y = [ 0.0009, 0.0044, 0.0175, 0.0540, ...
        0.1295, 0.2420, 0.3521, 0.3989, ...
        0.3521, 0.2420, 0.1295, 0.0540, ...
        0.0175, 0.0044, 0.0009 ] ;
end

%*****************************************************************************80
function F = eval_JF ( x )
  F = zeros(15,1) ;
  y = yvec();
  for i = 1 : 15
    F(i) = x(1) * exp ( - 0.5 * x(2) * ( 3.5 - 0.5 * ( i - 1 ) - x(3) )^2 ) - y(i);
  end
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros ( 15, 3 );
  y = yvec();
  for i = 1 : 15
    JF(i,1) = exp(-0.5 * x(2) * (-3 - x(3)) ^ 2);
    JF(i,2) = -0.5 * x(1) * (-3 - x(3)) ^ 2 * exp(-0.5 * x(2) * (-3 - x(3)) ^ 2);
    JF(i,3) = x(1) * x(2) * (-3 - x(3)) * exp(-0.5 * x(2) * (-3 - x(3)) ^ 2);
  end
end
