%*****************************************************************************80
%
%% Evaluates the objective function for problem 17.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = LSWood(what,x)
  global NF_eval NJF_eval ;

  switch what

  case {'name','title'}
    RES = 'The Wood Function.' ;

  case {'n','size'}
    RES = 4 ;

  case 'm'
    RES = 6 ;

  case {'init','start'}
    RES = [ -3.0, -1.0, -3.0, -1.0 ]' ;

  case 'solution'
    RES = [ 1.0, 1.0, 1.0, 1.0 ]' ;

  case 'F'
    NF_eval = NF_eval+1 ;
    RES = eval_F( x ) ;

  case 'JF'
    NJF_eval = NJF_eval+1;
    RES = eval_JF(x ) ;

  otherwise
    error(['The Wood Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function F = eval_F ( x )
  f1 = x(2) - x(1) * x(1);
  f2 = 1.0 - x(1);
  f3 = x(4) - x(3) * x(3);
  f4 = 1.0 - x(3);
  f5 = x(2) + x(4) - 2.0;
  f6 = x(2) - x(4);
  F  = [ f1 ; f2 ; f3 ; f4 ; f5 ; f6 ] ;
end

%*****************************************************************************80
function JF = eval_JF ( x )
  JF = zeros ( 6, 4 );

  JF(1,1) = -2*x(1) ;
  JF(1,2) = 1 ;

  JF(2,1) = -1 ;

  JF(3,3) = -2*x(3) ;
  JF(3,4) = 1 ;

  JF(4,3) = -1 ;

  JF(5,2) = 1 ;
  JF(5,4) = 1 ;

  JF(6,2) = 1 ;
  JF(6,4) = -1 ;
end
