function [ierr,X,F,NG] = GradientMinimization( fun, X0, varargin )
  ierr = 0 ; % ok found solution
  fX0  = feval( fun, 'f', X0 ) ;
  opts = { 'tol', 'linesearch', 'maxiter', 'c1', 'c2', 'tau', 'lmin', 'lmax', 'f0' };
  defs = {  1E-8,            1,       100, 1E-3,  0.5,   0.5,   1E-8,   1E+8, fX0 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  TOL        = OPTS.tol ;
  tau        = OPTS.tau ;
  c1         = OPTS.c1 ;
  c2         = OPTS.c2 ;
  maxiter    = OPTS.maxiter ;
  linesearch = OPTS.linesearch ;
  
  if any(isnan(X0))
    disp(X0) ;
    ierr = 1 ;
    fprintf(1, 'GradientMinimization: Bad initial point!' ) ;
    return ;
  end
  
  if linesearch < 1 | linesearch > 3
    ierr = 2 ;
    fprintf(1, 'GradientMinimization: lineserarch=%d must be in [1,3]',linesearch) ;
    return ;
  end

  xk = X0 ;
  F  = [] ;
  X  = [] ;
  NG = [] ;
  for k=1:maxiter
    % evaluate function and gradient
    fk = feval( fun, 'f',  xk ) ;
    Gk = feval( fun, 'Df', xk ) ;
    % save values
    X  = [ X xk ] ;
    F  = [ F fk ] ;
    NG = [ NG norm(Gk,'inf') ] ;
    % check if finished
    if ( NG(end) < TOL )
      return ;
    end
    % direction search
    d = -Gk' ;
    switch linesearch 
      case 1 ; alpha = ArmijoSimpleSearch( fun, xk, d, 'c1', c1, 'tau', tau ) ;
      case 2 ; alpha = ArmijoLineSearch( fun, xk, d, 'c1', c1 ) ;
      case 3 ; alpha = WolfeLineSearch( fun, xk, d, 'c1', c1, 'c2', c2 ) ;
    end
    if isnan( alpha )
      ierr = 3 ;
      return ;
    end
    xk = xk + alpha * d ;
  end
  ierr = 4 ;
  return ;
end

