%
% Function which plot iteration of secant methods
%
%  fun = function used by the secant methd
%  X   = vector with iteration history
%
%  xe  = exact solution
%
function secantPlotIteration( fun, X, xe )
  lw = {'r-','b-'} ;
  % evaluate X range
  xmin = min(X) ;
  xmax = max(X) ;

  % enlarge a bit the range for aestetical reason
  len  = xmax-xmin ;
  xmin = xmin - len/10 ;
  xmax = xmax + len/10 ;
  
  % sampling of the function for plotting
  XX   = [xmin:len/1000:xmax] ;
  YY   = feval(fun,XX) ;

  % evaluate y bounding box of the function
  ymin = min(YY) ;
  ymax = max(YY) ;
  
  % plot function
  plot(XX,YY,'g-','LineWidth',1.1);
  hold on;

  % plot axes 0 centered
  plot([xe;xe],[ymin;ymax],'k-','LineWidth',1);
  plot([xmin;xmax],[0;0],'k-','LineWidth',1);
  
  % evaluate function at iteration points
  fX = feval(fun, X) ;
  % vertical segment at iteration points
  for i=1:length(X)-2
    plot([X(i);X(i)],[0;fX(i)],'r--','LineWidth',1);
  end

  % plot secants from (x(i),f(X(i))) to (x(i+1),f(X(i+1)))
  % and intersecting x axes
  ii = 1 ;
  for i=1:length(X)-2
    ii = 3 - ii ;
    % secant
    plot([X(i);X(i+1)],[fX(i);fX(i+1)],lw{ii});
    % if secaant is on one side of x axex estend up to 0
    if fX(i)*fX(i+1) > 0 
      plot([X(i);X(i+2)],[fX(i);0],lw{ii});          
    end
  end
  axis on ;
  hold off;
end
